/* Copyright (c) 2015-present, salesforce.com, inc. All rights reserved
  Licensed under BSD 3-Clause - see LICENSE.txt or git.io/sfdc-license */

import { FocusVisibleMixin, ReactiveElement, DelegateFocusMixin } from 'elix';
import { defaultState, state, setState, template, render, firstRender } from 'elix/src/base/internal';
import { nanoid } from 'nanoid';
import css from './input-radio.css';
import common from '@salesforce-ux/sds-common/common.module.css';
import html from './input-radio.html';
import { templateFrom } from 'elix/src/core/htmlLiterals';
import { standardBooleanAttributes } from 'elix/src/core/AttributeMarshallingMixin';
import { delegateAttribute, reflectAttribute, dispatchCustomEvent } from '@salesforce-ux/sds-utils';

/**
 * Extend Elix's standard boolean attributes definition
 */
Object.assign(standardBooleanAttributes, {
  required: true,
  error: true,
});

const Base = DelegateFocusMixin(FocusVisibleMixin(ReactiveElement));

export default class C360InputRadio extends Base {
  get [defaultState]() {
    return Object.assign(super[defaultState], {
      id: `input-radio-${nanoid(10)}`,
      role: 'radio',
      name: '',
      value: '',
      size: 'large',
    });
  }

  get id() {
    return this[state].id;
  }

  set id(id) {
    this[setState]({ id });
  }

  get name() {
    return this[state].name;
  }

  set name(name) {
    this[setState]({ name });
  }

  get value() {
    return this[state].value;
  }

  set value(value) {
    this[setState]({ value });
  }

  get checked() {
    return this[state].checked;
  }

  set checked(checked) {
    this[setState]({ checked });
  }

  get disabled() {
    return this[state].disabled;
  }

  set disabled(disabled) {
    this[setState]({ disabled });
  }

  get size() {
    return this[state].size;
  }

  set size(size) {
    this[setState]({ size });
  }

  get role() {
    return this[state].role;
  }

  set role(role) {
    this[setState]({ role });
  }

  get required() {
    return this[state].required;
  }

  set required(required) {
    this[setState]({ required });
  }

  get error() {
    return this[state].error;
  }

  set error(error) {
    this[setState]({ error });
  }

  [render](changed) {
    super[render](changed);
    const { id, name, value, checked, disabled, size, role, required, error } = this[state];
    const radio = this.shadowRoot.querySelector('[part~="radio"]');
    const label = this.shadowRoot.querySelector('[part~="label"]');

    if (this[firstRender] && radio) {
      radio.addEventListener('change', (event) => this._handleChange(event));
      radio.addEventListener('focus', (event) => this._handleFocus(event));
      radio.addEventListener('blur', (event) => this._handleBlur(event));

      if (this.checked) {
        radio.setAttribute('checked', '');
      }
    }

    if (changed.id) {
      if (id) {
        delegateAttribute(this, radio, 'id', id);
        label.setAttribute('for', id);
      }
    }

    if (changed.name) {
      if (name) {
        reflectAttribute(this, 'name', name);
        radio.setAttribute('name', name);
      }
    }

    if (changed.value) {
      if (value) {
        reflectAttribute(this, 'value', value);
        radio.setAttribute('value', value);
      }
    }

    if (changed.checked) {
      reflectAttribute(this, 'checked', checked);
      reflectAttribute(radio, 'checked', checked);
      radio.checked = checked;
    }

    if (changed.disabled) {
      reflectAttribute(this, 'disabled', disabled);
      reflectAttribute(radio, 'disabled', disabled);
    }

    if (changed.size) {
      reflectAttribute(this, 'size', size);
    }

    if (changed.role) {
      reflectAttribute(this, 'role', role);
    }

    if (changed.required) {
      reflectAttribute(this, 'required', required);
      reflectAttribute(radio, 'required', required);
    }

    if (changed.error) {
      reflectAttribute(this, 'error', error);
      reflectAttribute(radio, 'error', error);
    }
  }

  /**
   * Dispatches change event
   */
  _handleChange() {
    const radio = this.shadowRoot.querySelector('[part~="radio"]');
    this[setState]({ checked: radio.checked });
    dispatchCustomEvent(this, 'change');
  }

  /**
   * Dispatches focus event
   */
  _handleFocus() {
    dispatchCustomEvent(this, 'focus');
  }

  /**
   * Dispatches blur event
   */
  _handleBlur() {
    dispatchCustomEvent(this, 'blur');
  }

  get [template]() {
    const result = templateFrom.html`
      <style>
        ${common}
        ${css}
      </style>
      ${html}
    `;
    return result;
  }
}
